#version 330 core
out vec4 FragColor;
in vec2 TexCoords;

uniform sampler2D screenTexture;
uniform sampler2D velocityTexture;

void main() {
    vec2 velocity = texture(velocityTexture, TexCoords).xy;
    velocity *= 0.05;

    vec3 color = texture(screenTexture, TexCoords).rgb;
    int samples = 8;
    vec3 accumulatedColor = color;
    float weightSum = 1.0;  // Start with the central sample's weight

    if (length(velocity) > 0.001) { // Apply blur only if there is noticeable motion
        for (int i = 1; i < samples; ++i) {
            float scale = float(i) / float(samples);
            vec2 offset = velocity * scale;
            vec2 pos1 = TexCoords + offset;
            vec2 pos2 = TexCoords - offset;

            // Clamp coordinates to prevent sampling outside the texture
            pos1 = clamp(pos1, 0.0, 1.0);
            pos2 = clamp(pos2, 0.0, 1.0);

            accumulatedColor += texture(screenTexture, pos1).rgb;
            accumulatedColor += texture(screenTexture, pos2).rgb;
            weightSum += 2.0;  // Increment weight for both added samples
        }
    }

    FragColor = vec4(accumulatedColor / weightSum, 1.0);
}