
/*
* Copyright 2023 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the GCG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;

out VertexData {
#if defined(BLOBBY)
	vec3 object_position;
	vec3 camera_position_object;
#endif
	vec3 position_world;
	vec3 prev_position_world_proj;
	vec3 position_world_proj;
	vec3 normal_world;
	vec2 uv;
} vert;

#if defined(BLOBBY)
	uniform vec3 camera_world;
#endif

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

uniform mat4 prevViewProjMatrix;

#ifdef ANIM
	uniform uint time;
#endif

void main() {
	vert.normal_world = normalMatrix * normal;
	vert.uv = uv;

	vec3 position_ = position;

	#ifdef ANIM
		// We add a simple sine wave animation to the y axis of the mesh based on the time and position
		#if defined(ANIM_SM)
			#define POS_SCALE 10.0
			#define FULL_SCALE 0.01
		#else
			#define POS_SCALE 0.1
			#define FULL_SCALE 0.1
		#endif

		#define TIME_SCALE 0.001

		float time_ = float(time);
		float offset = FULL_SCALE * sin(time_ * TIME_SCALE + position.x * POS_SCALE + position.y * POS_SCALE + position.z * POS_SCALE);
		position_ += vec3(0, offset * 10.0, 0);

		// Transform normals
		// TODO: Check if this is correct
		vert.normal_world = normalMatrix * (normal + vec3(0, 1, 0) * offset);
	#endif

	vec4 position_world_ = modelMatrix * vec4(position_, 1);
	vert.position_world = position_world_.xyz;

	gl_Position = viewProjMatrix * position_world_;

    // Calculate current position
    vert.position_world_proj = gl_Position.xyz;

	// Calculate previous position
	vec4 prev_position_world_ = prevViewProjMatrix * position_world_;
	vert.prev_position_world_proj = prev_position_world_.xyz;

	#if defined(BLOBBY)
		vert.object_position = position_;
		vert.camera_position_object = (inverse(modelMatrix) * vec4(camera_world, 1)).xyz;
	#endif
}
